@@ -1,4 +1,4 @@
-// Copyright (C) 2021  Tassilo Horn <tsdh@gnu.org>
+// Copyright (C) 2021-2022  Tassilo Horn <tsdh@gnu.org>
 //
 // This program is free software: you can redistribute it and/or modify it
 // under the terms of the GNU General Public License as published by the Free
@@ -15,6 +15,10 @@
 
 //! The `swayrd` binary.
 
+use env_logger::Env;
+
 fn main() {
-    swayr::demon::run_demon();
+    env_logger::Builder::from_env(Env::default().default_filter_or("warn"))
+        .init();
+    swayr::daemon::run_daemon();
 }
